/*

  This file is a part of GRALESHELL, a program to facilitate the simulation
  and inversion of gravitational lenses, based on the GRALE library.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include <mpi.h>
#include "cmdimageplanempi.h"
#include "graleshellinstance.h"
#include "graleshellimageplane.h"
#include "mainmpi.h"

#include "debugnew.h"

using namespace shellp;

CmdImagePlaneMPI::CmdImagePlaneMPI(const std::string &cmdname) : CmdImagePlane(cmdname)
{
	setDescription("Create image plane based upon the current lens and source plane. Calculations are done distributed using MPI.");
}

CmdImagePlaneMPI::~CmdImagePlaneMPI()
{
}

bool CmdImagePlaneMPI::execute()
{
	GraleShellInstance *gsinst = (GraleShellInstance *)ShellInstance::getInstance();
	grale::GravitationalLens *lens;
	grale::SourcePlane *srcplane;
	
	if ((lens = gsinst->GetLens()) == 0)
	{
		setErrorString("A lens must have been created first");
		return false;
	}
	if ((srcplane = gsinst->GetSourcePlane()) == 0)
	{
		setErrorString("A source plane must have been created first");
		return false;
	}

	// Send the command and transmit the parameters to the other ranks
	int command = GRALESHELL_MPICOMMAND_IMAGEPLANE;
	MPI_Bcast(&command, 1, MPI_INT, 0, MPI_COMM_WORLD);

	grale::Vector2D<double> bottomleft(GetX1(),GetY1());
	grale::Vector2D<double> topright(GetX2(),GetY2());
	
	GraleShellImagePlane *imgplane = new GraleShellImagePlane(lens, srcplane);

	if (!imgplane->initMPI(bottomleft,topright,GetNumX(),GetNumY()))
	{
		setErrorString("Couldn't initialize the lens plane: " + imgplane->getErrorString());
		delete imgplane;
		return false;
	}
	
	gsinst->SetImagePlane(imgplane);
	
	return true;
}

